//
//  ViewController.m
//  TextClassTester
//
//  Created by narita on 2016/09/08.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "ViewController.h"
#import "MT1STextFieldCellExtention.h"

@implementation ViewController

- (void)viewDidLoad 
{
    [super viewDidLoad];

    // Do any additional setup after loading the view.
}

- (void)setRepresentedObject:(id)representedObject {
    [super setRepresentedObject:representedObject];

    // Update the view, if already loaded.
}

- (void)controlTextDidChange:(NSNotification *)obj
{
    // 1文字入力毎に行の高さの再計算を行う。    
    NSTextField* theTextFiled = obj.object;
    
    if( [theTextFiled isKindOfClass:[NSTextField class]] )
    {
        CGFloat theOldHeight = theTextFiled.frame.size.height;
        CGFloat theNewHeight = [theTextFiled calcBoundingHeightWithFixedCurrentWidth]; 
        
        theTextFiled.frame = NSMakeRect(
                                        theTextFiled.frame.origin.x,
                                        theTextFiled.frame.origin.y + (theOldHeight - theNewHeight),
                                        theTextFiled.frame.size.width,
                                        theNewHeight);
        
        [theTextFiled layout];
    }
}

- (BOOL)control:(NSControl*)control textView:(NSTextView*)textView doCommandBySelector:(SEL)commandSelector
{
    BOOL theResult = NO;
    
    if( commandSelector == @selector(insertNewline:) )
    {
        [textView insertNewlineIgnoringFieldEditor:self];
        
        theResult = YES;
    }
    else if( commandSelector == @selector(insertTab:) )
    {
        NSLog(@"%s::insertTab", __func__);
        
        theResult = YES;
    }
    else if( commandSelector == @selector(moveRight:) )
    {
        theResult = [self moveRightWithControl:control textView:textView];
    }
    else if( commandSelector == @selector(moveLeft:) )
    {
        theResult = [self moveLeftWithControl:control textView:textView];        
    }
    else if( commandSelector == @selector(moveDown:) )
    {
        theResult = [self moveDownWithControl:control textView:textView];        
    }
    else if( commandSelector == @selector(moveUp:) )
    {
        theResult = [self moveUpWithControl:control textView:textView];        
    }
    
    return theResult;
}


- (BOOL) moveRightWithControl:(NSControl*)control textView:(NSTextView*)textView
{
    BOOL theResult = NO;

    if( [textView isSelectedRangeLastPosition] )// 文末にカーソルがある
    {
        // 次のNSTextFieldにフォーカスを移し、カーソル位置を先頭に持って行く        
        NSTextField* theTextFiled = (NSTextField*)control.nextKeyView;
        
        if( [theTextFiled isKindOfClass:[NSTextField class]] )
        {            
            [theTextFiled becomeFirstResponder];
            [theTextFiled.currentEditor moveToBeginningOfDocument:nil];
            theResult = YES;
        }
    }
    
    return theResult;
}

- (BOOL) moveLeftWithControl:(NSControl*)control textView:(NSTextView*)textView
{
    BOOL theResult = NO;
    
    if( [textView isSelectedRangeTopPosition] )// 文頭にカーソルがある
    {
        // 前のNSTextFieldにフォーカスを移し、カーソル位置を末尾に持って行く        
        NSTextField* theTextFiled = (NSTextField*)control.previousKeyView;
        
        if( [theTextFiled isKindOfClass:[NSTextField class]] )
        {            
            [theTextFiled becomeFirstResponder];
            [theTextFiled.currentEditor moveToEndOfDocument:nil];
            theResult = YES;
        }
    }
    
    return theResult;
}



- (BOOL) moveDownWithControl:(NSControl*)control textView:(NSTextView*)textView
{
    BOOL theResult = NO;
    
    // キャレットが、最終行にあるかの判定
    if( [textView isSelectedRangeLastLine] )
    {        
        // 次のNSTextFieldにフォーカスを移し、カーソル位置をできるだけ合わせる
        NSTextField* theTextFiled = (NSTextField*)control.nextKeyView;
        
        if( [theTextFiled isKindOfClass:[NSTextField class]] )
        {      
            NSRect  theCurrentRect  = [textView rectForCharacterRange:textView.selectedRange];

            [theTextFiled becomeFirstResponder];

            // 最初の行でありかつ、キャレットのX軸が大体同じ位置に移動
            [textView selectFirstLineAt:theCurrentRect.origin.x];
                        
            theResult = YES;
        }
    }
    
    return theResult;
}


- (BOOL) moveUpWithControl:(NSControl*)control textView:(NSTextView*)textView
{
    BOOL theResult = NO;
            
    // キャレットが、最初の行にあるかの判定
    if( [textView isSelectedRangeFirstLine] )
    {        
        // 次のNSTextFieldにフォーカスを移し、カーソル位置をできるだけ合わせる
        NSTextField* theTextFiled = (NSTextField*)control.previousKeyView;
        
        if( [theTextFiled isKindOfClass:[NSTextField class]] )
        {       
            NSRect  theCurrentRect  = [textView rectForCharacterRange:textView.selectedRange];

            [theTextFiled becomeFirstResponder];
            
            // 最終行でありかつ、キャレットのX軸が大体同じ位置に移動
            [textView selectLastLineAt:theCurrentRect.origin.x];
            
            theResult = YES;
        }
    }
        
    return theResult;
}

@end
